@@ -34,7 +34,7 @@
 #' higher-level functions. The handle to the database can be closed
 #' by passing it to `close`.
 #'
-#' @param path The path to a world folder. If the path does not exist, it is 
+#' @param path The path to a world folder. If the path does not exist, it is
 #'   assumed to be the base name of a world folder in the local minecraftWorlds
 #'   directory.
 #' @param create_if_missing Create world database if it doesn't exist.
@@ -46,9 +46,13 @@
 #' @param cache_capacity Internal leveldb option
 #' @param bloom_filter_bits_per_key Internal leveldb option
 #' @param compression_level Internal leveldb option
+#' @param compact Compact database before closing.
+#' @param con An database object created by bedrockdb.
+#' @param ... arguments passed to or from other methods.
+#' @param x An object.
 #'
 #' @return On success, `bedrockdb` returns an R6 class of type 'bedrockdb'.
-#'         
+#'
 #' @export
 #' @examples
 #' # open an example works and get all keys
@@ -60,47 +64,74 @@
 #' \dontrun{
 #'
 #' # open a world in the minecraftWorlds folder using a world id.
-#' db <- bedrockdb("lrkkYFpUABA=") 
+#' db <- bedrockdb("lrkkYFpUABA=")
 #' # do something with db ...
 #' close(db)
 #'
 #' # open a world using absolute path
-#' db <- bedrockdb("C:\\\\minecraftWorlds\\\\my_world") 
+#' db <- bedrockdb("C:\\\\minecraftWorlds\\\\my_world")
 #' # do something with db ...
 #' close(db)
 #' }
 
-bedrockdb <- function(path, create_if_missing = FALSE, error_if_exists = NULL, paranoid_checks = NULL, 
-    write_buffer_size = 4194304L, max_open_files = NULL, block_size = NULL, 
-    cache_capacity = 41943040L, bloom_filter_bits_per_key = 10L, compression_level = -1L) {
-    R6_bedrockdb$new(path, create_if_missing, error_if_exists, paranoid_checks, write_buffer_size, 
-        max_open_files, block_size, cache_capacity, bloom_filter_bits_per_key, compression_level)
+bedrockdb <- function(path,
+                      create_if_missing = FALSE,
+                      error_if_exists = NULL,
+                      paranoid_checks = NULL,
+                      write_buffer_size = 4194304L,
+                      max_open_files = NULL,
+                      block_size = 163840L,
+                      cache_capacity = 83886080L,
+                      bloom_filter_bits_per_key = 10L,
+                      compression_level = -1L) {
+    R6_bedrockdb$new(path,
+                     create_if_missing,
+                     error_if_exists,
+                     paranoid_checks,
+                     write_buffer_size,
+                     max_open_files,
+                     block_size,
+                     cache_capacity,
+                     bloom_filter_bits_per_key,
+                     compression_level)
 }
 
 #' @export
-#' @param compact Compact database before closing.
-#' @param con An database object created by bedrockdb.
-#' @param ... arguments passed to or from other methods.
-#'
 #' @rdname bedrockdb
 close.bedrockdb <- function(con, compact = FALSE, ...) {
-    if(isTRUE(compact)) {
+    if (isTRUE(compact)) {
         inform("Compacting database...")
         con$compact_range()
     }
-    con$close()
+    con$close(...)
+}
+
+#' @export
+#' @rdname bedrockdb
+is_bedrockdb <- function(x) {
+    inherits(x, "bedrockdb")
 }
 
 #' @importFrom R6 R6Class
-R6_bedrockdb <- R6::R6Class("bedrockdb", public = list(db = NULL, path = NULL, levelname = NULL, 
+R6_bedrockdb <- R6::R6Class("bedrockdb", public = list(
+    db = NULL,
+    path = NULL,
+    levelname = NULL,
+    leveldat = NULL,
+    leveldat_is_dirty = FALSE,
+    unique_id = NULL,
     initialize = function(path, ...) {
         path <- fs::path_real(.fixup_path(path))
-        namefile <- fs::path(path, "levelname.txt")
-        self$levelname <- readr::read_file(namefile)
+        dat <- read_leveldat(path)
+        self$levelname <- payload(dat$LevelName)
         self$path <- fs::path(path, "db")
         self$db <- bedrock_leveldb_open(self$path, ...)
+        self$leveldat <- dat
     },
     close = function(error_if_closed = FALSE) {
+        if (self$leveldat_is_dirty) {
+            write_leveldat(self$leveldat, fs::path_dir(self$path))
+        }
         ret <- bedrock_leveldb_close(self$db, error_if_closed)
         invisible(ret)
     },
@@ -123,7 +154,7 @@ R6_bedrockdb <- R6::R6Class("bedrockdb", public = list(db = NULL, path = NULL, l
     },
     mget_prefix = function(starts_with, readoptions = NULL) {
         bedrock_leveldb_mget_prefix(self$db, starts_with, readoptions)
-    },    
+    },
     put = function(key, value, writeoptions = NULL) {
         bedrock_leveldb_put(self$db, key, value, writeoptions)
         invisible(self)
@@ -132,8 +163,9 @@ R6_bedrockdb <- R6::R6Class("bedrockdb", public = list(db = NULL, path = NULL, l
         bedrock_leveldb_mput(self$db, keys, values, writeoptions)
         invisible(self)
     },
-    delete = function(keys, report = FALSE, readoptions = NULL, writeoptions = NULL) {
-        bedrock_leveldb_delete(self$db, keys, report, readoptions, 
+    delete = function(keys, report = FALSE, readoptions = NULL,
+                      writeoptions = NULL) {
+        bedrock_leveldb_delete(self$db, keys, report, readoptions,
             writeoptions)
     },
     exists = function(key, readoptions = NULL) {
@@ -160,10 +192,22 @@ R6_bedrockdb <- R6::R6Class("bedrockdb", public = list(db = NULL, path = NULL, l
     compact_range = function(start = NULL, limit = NULL) {
         bedrock_leveldb_compact_range(self$db, start, limit)
         invisible(self)
+    },
+    create_unique_ids = function(n) {
+        if (is_null(self$unique_id)) {
+            cnt <- payload(self$leveldat$worldStartCount %||% nbt_long(0))
+            self$leveldat$worldStartCount <- nbt_long(cnt - 1)
+            self$leveldat_is_dirty <- TRUE
+            self$unique_id <- (cnt - 2^32) * 2^32
+        }
+        ret <- self$unique_id + seq_len(n)
+        self$unique_id <- self$unique_id + n
+        ret
     }
 ))
 
-R6_bedrockdb_iterator <- R6::R6Class("bedrockdb_iterator", public = list(it = NULL, 
+R6_bedrockdb_iterator <- R6::R6Class("bedrockdb_iterator", public = list(
+    it = NULL,
     initialize = function(db, readoptions) {
         self$it <- bedrock_leveldb_iter_create(db, readoptions)
     },
@@ -202,8 +246,10 @@ R6_bedrockdb_iterator <- R6::R6Class("bedrockdb_iterator", public = list(it = NU
     }
 ))
 
-R6_bedrockdb_writebatch <- R6::R6Class("bedrockdb_writebatch", public = list(ptr = NULL, 
-    db = NULL, initialize = function(db) {
+R6_bedrockdb_writebatch <- R6::R6Class("bedrockdb_writebatch", public = list(
+    ptr = NULL,
+    db = NULL,
+    initialize = function(db) {
         self$db <- db
         self$ptr <- bedrock_leveldb_writebatch_create()
     },
@@ -230,7 +276,7 @@ R6_bedrockdb_writebatch <- R6::R6Class("bedrockdb_writebatch", public = list(ptr
     mdelete = function(keys) {
         bedrock_leveldb_writebatch_mdelete(self$ptr, keys)
         invisible(self)
-    },    
+    },
     write = function(writeoptions = NULL) {
         bedrock_leveldb_write(self$db, self$ptr, writeoptions)
         invisible(self)
